/* ------------------------------------------------------------------
                              ht16k33.h

     Header fuer Softwaremodul fuer LED Anzeigentreiber HT16K33, hier
     fuer max. 8 stellige 7-Segment LED-Anzeige

     benoetigt i2c.c / i2c.h

     MCU   :  CH32V003
     Takt  :

     13.05.2025  R. Seelig
   ------------------------------------------------------------------ */

/*
                  CH32V003 A4M6                                        CH32V003F4P6
                  +-----------+                                      +--------------+
        PC1 / sda |1    C   16| PC0                         PD4 / a7 | 1         20 | PD3 / a4
        PC2 / scl |2    H   15| Vdd                   PD5 / a5 / utx | 2    C    19 | PD2 / a3
              PC3 |3    3   14| gnd                   PD6 / a6 / urx | 3    H    18 | PD1 / swio
              PC4 |4    2   13| PA2 / osc0 / a0           nrst / PD7 | 4    3    17 | PC7 / miso
       PC6 / mosi |5    V   12| PA1 / osc1 / a1      PA1 / osc0 / a0 | 5    2    16 | PC6 / mosi
       PC7 / miso |6    0   11| nrst / PD7           PA2 / osc1 / a1 | 6    V    15 | PC5 / sck / scl
       PD1 / swio |7    0   10| PD6 / a6 / urx                   gnd | 7    0    14 | PC4 / a2
         PD4 / a7 |8    3    9| PD5 / a5 / utx                   PD0 | 8    0    13 | PC3
                  +-----------+                                  Vdd | 9    3    12 | PC2 / scl
                                                                 PC0 | 10        11 | PC1 / sda
                                                                     +--------------+

                      HT16K33
                  +--------------+
              GND | 1         28 | +Vdd
        COM0 / AD | 2         27 | SDA
       COM1 / KS0 | 3    H    26 | SCL
       COM2 / KS1 | 4    T    25 | ROW0 / A2
       COM3 / KS2 | 5         24 | ROW1 / A1
             COM4 | 6    1    23 | ROW2 / A0
             COM5 | 7    6    22 | ROW3 / K1
             COM6 | 8    K    21 | ROW4 / K2
             COM7 | 9    3    20 | ROW5 / K3
ROW15 / K13 / INT | 10   3    19 | ROW6 / K4
      ROW14 / K12 | 11        18 | ROW7 / K5
      ROW13 / K11 | 12        17 | ROW8 / K6
      ROW12 / K10 | 13        16 | ROW9 / K7
       ROW11 / k9 | 14        15 | ROW10 / K8
                  +--------------+

*/

#ifndef in_ht16k33
  #define in_ht16k33

  #include <string.h>
  #include "ch32fun.h"
  #include "ch32v003_gpio.h"

  #include "i2c_sw.h"


  #define ascofs                0x20           // Asciitabelle der Bitmaps faengt mit Leerzeichen
                                               // (Ascii-Code 32d = 0x20) an
  #define zifofs                0x10           // Ziffernoffset des Codetabelle fuer Ziffern

  #define ht16k_addr            0xe0           // I2C-Adresse des 7-Segmenttreibers

  #define bufsize               8              // Bytegroesse des Displayframebuffers


  extern uint8_t fbuffer[bufsize];
  extern const uint8_t bmps7asc[];


/* --------------------------------------------------------
                         Prototypen
   -------------------------------------------------------- */
uint8_t ht16k_wrcmd(uint8_t val);
uint8_t ht16k_wr2ram(uint8_t val, uint8_t segaddr);
void ht16k_wrbuffer(void);
uint8_t ht16k_setbrightness(uint8_t val);
void ht16k_init(void);
void ht16k_setdez(int32_t value, uint8_t pos, uint8_t nozero);
void ht16k_sethex(int32_t value, uint8_t pos, uint8_t nozero);
void ht16k_setdp(uint8_t pos, uint8_t enable);
void ht16k_puts(char *s, uint8_t anz);


#endif
