/* -------------------------------------------------------
                         toene_demo.c

     Spielt einen Notenstring ab, Lautsprecher an PD4
     (in toene.h festgelegt)

     Verwendet Timer1 zur Taktgenerierung der Frequenzen

     MCU : ch32v003

     07.05.2025  R. Seelig
   -------------------------------------------------------- */

/*

                  CH32V003 A4M6                                        CH32V003F4P6

                  +-----------+                                      +--------------+
        PC1 / sda |1    C   16| PC0                         PD4 / a7 | 1         20 | PD3 / a4
        PC2 / scl |2    H   15| Vdd                   PD5 / a5 / utx | 2    C    19 | PD2 / a3
              PC3 |3    3   14| gnd                   PD6 / a6 / urx | 3    H    18 | PD1 / swio
              PC4 |4    2   13| PA2 / osc0 / a0           nrst / PD7 | 4    3    17 | PC7 / miso
       PC6 / mosi |5    V   12| PA1 / osc1 / a1      PA1 / osc0 / a0 | 5    2    16 | PC6 / mosi
       PC7 / miso |6    0   11| nrst / PD7           PA2 / osc1 / a1 | 6    V    15 | PC5 / sck / scl
       PD1 / swio |7    0   10| PD6 / a6 / urx                   gnd | 7    0    14 | PC4 / a2
         PD4 / a7 |8    3    9| PD5 / a5 / utx                   PD0 | 8    0    13 | PC3
                  +-----------+                                  Vdd | 9    3    12 | PC2 / scl
                                                                 PC0 | 10        11 | PC1 / sda
                                                                     +--------------+

*/

#include "ch32fun.h"
#include "ch32v003_gpio.h"

#include "toene.h"

// Jingle
char jingle01[] = { "g8g8g8h8o+e4d3" };

// Schneewalzer
char schneew [] = { "c4d4e2g4e2g4e1d4e4f2g4f2g4f1g4a4h2+f4-h2+f4-h1a4h4+c2e4c2-a4g1e4g4+c1-h1+d1c1-a2+c4-a2+c4-a1" };


/* --------------------------------------------------------
                             main
   -------------------------------------------------------- */
int main(void)
{
  SystemInit();

  notenplay_init();

  sound_on();
  // Schneewalzer
  playstring(&schneew[0]);
  sound_off();
  delay(200);

  // Jingle
  playstring(&jingle01[0]);

  sound_off();

  while(1);                   // Endlosschleife
}
