/* -----------------------------------------------------------------------------------
                                      spi.c

     SPI-Interface fuer die meisten Anwendungsfaelle geeignet (insbesondere zum
     Ansteuern von GLC-Displays)

     MCU   : CH32V003
     Takt  :

     22.05.2025  R. Seelig
   ----------------------------------------------------------------------------------- */

#include "spi.h"

/* -------------------------------------------------------------
   spi_init

      initialisiert SPI-Schnittstelle (vorzugsweise zur
      Benutzung mit SPI LC-Displays)
   ------------------------------------------------------------- */
void spi_init(void)
{
  RCC->APB2PCENR |= RCC_APB2Periph_SPI1;                // SPI einschalten
  delay(250);

  funPinMode( PC6, GPIO_CFGLR_OUT_50Mhz_AF_PP );        // alternate Funktion fuer PC6-mosi
  funPinMode( PC5, GPIO_CFGLR_OUT_50Mhz_AF_PP );        // alternate Funktion fuer PC5-clk

  SPI1->CTLR1 =
          SPI_NSS_Soft | SPI_CPHA_1Edge | SPI_CPOL_Low | SPI_DataSize_8b |
          SPI_Mode_Master | SPI_Direction_1Line_Tx |
          SPI_BaudRatePrescaler_2;

  SPI1->CTLR1 |= CTLR1_SPE_Set;         // SPI aktivieren

}



/* -------------------------------------------------------------
   spi_lcdout

      Byte ueber SPI senden
      data ==> zu sendendes Datum
   ------------------------------------------------------------- */
void spi_out(uint8_t data)
{
  while(!(SPI1->STATR & SPI_STATR_TXE));   // warten bis Sendepuffer leer ist

  SPI1->DATAR = data;                      // Zeichen senden

  while(SPI1->STATR & SPI_STATR_BSY);      // warten bis Uebertragung beendet ist
}

