/* -------------------------------------------------------
                         tm16xx_demo.c

     Demoprogramm 4-stellige 7-Segmentanzeige
     7-Segmentdekodeauswahl in tm16xx.h (TM1637, TM1650,
     TM1651)

     MCU   :  CH32V003

     24.03.2025  R. Seelig
   ------------------------------------------------------ */


#include "ch32fun.h"
#include "ch32v003_gpio.h"
#include "tm16xx.h"

/*

   Anschlussbelegung:
 ------------------------------------------------------

     TM16xx           CH32V003
     -------------------------
       SCL   .......... PC2
       SDA   .......... PC1

   Segmentbelegung der Anzeige:

       a
      ---
   f | g | b            Segment |  a  |  b  |  c  |  d  |  e  |  f  |  g  | Doppelpunkt (nur fuer POS1) |
      ---               ---------------------------------------------------------------------------------
   e |   | c            Bit-Nr. |  0  |  1  |  2  |  3  |  4  |  5  |  6  |              7              |
      ---
       d

*/


uint8_t laufmust1[] =
{
  0x11, 0x21, 0x31, 0x41,   0x42,
  0x47, 0x37, 0x27, 0x17,   0x15,
  0x14, 0x24, 0x34, 0x44,   0x43,
  0x47, 0x37, 0x27, 0x17,   0x16,  0x00
};

uint8_t gradc = 0x63;             // Bitmuster fuer hochgestelltes "o"

/* -------------------------------------------------------
                         playmuster

     zeigt ein Lauflichtmuster auf der Anzeige
   ------------------------------------------------------- */
void playmuster(uint8_t *muster, uint16_t speed)
{
  uint8_t c, anzp, segp;

  while(1)
  {
    c= *muster;
    if (!(c)) return;
    muster++;
    tm16_clear();
    anzp= ((c >> 4) & 0x0f) - 1;
    segp= (c & 0x0f) - 1;
    tm16_setseg(anzp, segp);
    delay(speed);
  };
}

/* -------------------------------------------------------
                             MAIN
   ------------------------------------------------------- */
int main()
{
  int16_t b;

  SystemInit();

  tm16_init();

  while(1)
  {
    tm16_setdez2(2,25);
    tm16_setbmp(3, gradc);
    delay(1000);

    playmuster(&laufmust1[0], 50);

    b= 0;
    tm16_sethex(b);

    do                                         // hexadezimaler Aufwaertszaehler
    {
      tm16_sethex(b);
      delay(1000);
      b++;
    } while(b< 0x10);
    tm16_sethex(b);
    delay(1000);

    tm16_clear();
    delay(300);

    b= 10;
    tm16_setdez(b,0);
    delay(500);

    do                                        // dezimaler Countdown
    {
      tm16_setdez(b,0);
      delay(500);
      tm16_setdez(b,3);
      delay(500);
      b--;
    } while(b);
    tm16_setdez(b,0);
    delay(1000);
  }
}

