/* --------------------------------------------------------
                        uart_input_demo.c

     Demoprogramm fuer Eingaben an der seriellen
     Schnittstelle

     TxD an PD5, RxD an PD6

     MCU : ch32v003

     08.04.2025  R. Seelig
   -------------------------------------------------------- */

/*
                  CH32V003 A4M6                                        CH32V003F4P6

                  +-----------+                                      +--------------+
        PC1 / sda |1    C   16| PC0                         PD4 / a7 | 1         20 | PD3 / a4
        PC2 / scl |2    H   15| Vdd                   PD5 / a5 / utx | 2    C    19 | PD2 / a3
              PC3 |3    3   14| gnd                   PD6 / a6 / urx | 3    H    18 | PD1 / swio
              PC4 |4    2   13| PA2 / osc0 / a0           nrst / PD7 | 4    3    17 | PC7 / miso
       PC6 / mosi |5    V   12| PA1 / osc1 / a1      PA1 / osc0 / a0 | 5    2    16 | PC6 / mosi
       PC7 / miso |6    0   11| nrst / PD7           PA2 / osc1 / a1 | 6    V    15 | PC5 / sck / scl
       PD1 / swio |7    0   10| PD6 / a6 / urx                   gnd | 7    0    14 | PC4 / a2
         PD4 / a7 |8    3    9| PD5 / a5 / utx                   PD0 | 8    0    13 | PC3
                  +-----------+                                  Vdd | 9    3    12 | PC2 / scl
                                                                 PC0 | 10        11 | PC1 / sda
                                                                     +--------------+
*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "ch32fun.h"
#include "ch32v003_gpio.h"

#include "my_printf.h"
#include "uart_read.h"

#define led_init()               PC3_output_init()
#define led_set()                PC3_set()
#define led_clr()                PC3_clr()

#define cnt_speed    500

#define baud         115200

#define printf       my_printf


/* ----------------------------------------------------------
                        my_putchar

     wird von my_printf aufgerufen. Hierauf gibt my_printf
     den Datenstream aus
   ---------------------------------------------------------- */
void my_putchar(char ch)
{
  uart_putchar(ch);
}

/* ----------------------------------------------------------
                              main
   ---------------------------------------------------------- */
int main(void)
{
  int         li;
  int         val1;

  char        mytext[120];
  char        myval[12];

  SystemInit();                 // 48 MHz intern, Clock fuer PORTA, PORTC, PORTD eingeschaltet

  li= F_CPU / 1000000;

  led_init();

  uart_init(baud);
  printf("\n\r--------------------------------------");
  printf("\n\rCH32V003 running at %d MHz\n\r", (uint16_t)li);
  printf("16 kByte Flash, 2 KByte RAM\n\n\r");
  printf("Baudrate: %d\n\n\r",baud);
  printf("08.04.2025  R. Seelig\n\r");
  printf("--------------------------------------\n\n\r");

  printf(" Demo fuer Eingaben ueber UART\n\r");
  printf(" Fehlerhafte Eingabe mit 'Entf' loeschen\n\r");
  printf(" -------------------------------------\n\n\r");

  while(1)
  {
    strcpy(mytext, "Hallo Welt");
    printf(" Texteingabe: ");
    uart_lineinput(mytext, stringinput, 100);

    printf("\n\r Die Eingabe war: %s\n\n\r", mytext);


    #if (enable_komma_convert == 1)
      strcpy(myval,"3.1415");
      printf("\n\r Eingabe Kommazahl: ");
      val1= uart_readkomma(myval, 10, &printfkomma);
      printf("\n\r Pseudokommazahl: %k\n\n\r", val1);
    #endif

    #if (enable_float_convert == 1)
      float       f;

      strcpy(myval,"1.414");
      printf("\n\r Eingabe Kommazahl(float): ");
      f= uart_readfloat(myval, 10);

      f= f * 3.3;
      f= f * 1000.0;

      printfkomma= 3;
      val1= f;
      printf("\n\r Rechnung (float-eingabe) * 3.3=  %k \n\n\r", val1);
    #endif

    #if (enable_int_convert == 1)
      int val2;

      strcpy(myval, "1");
      printf(" LED an/aus [0,1]: ");
      val1= uart_readint(myval, 1);
      if (val1== 0) { led_clr(); } else { led_set(); }
      printf("\n\n\r");

      strcpy(myval,"12");
      printf(" Integereingabe 1: ");
      val1= uart_readint(myval, 9);

      strcpy(myval,"18");
      printf("     Integereingabe 2: ");
      val2= uart_readint(myval, 9);

      printf("\n\r %d * %d = %d \n\n\r", val1, val2, val1 * val2);
    #endif

    #if (enable_hex_convert == 1)
      strcpy(myval,"a");
      printf(" Hexadezimaleingabe: ");
      val1= uart_readhex(myval, 8);
      printf("\n\r 0x%x = %dd = '%c'Ascii\n\n\r",val1, val1, (val1 & 0xff));
    #endif
  }
}
