/* --------------------------------------------------------
                        uart_read.h

     Stellt Eingabefunktionen ueber die serielle Schnitt-
     stelle zur Verfuegung. Eine gemeinsame Eingabefunktion
     stellt unterschiedliche Eingaben fuer

        - Texte
        - Integerzahlen
        - Kommazahlen
        - Hexadezimale Zahlen

     bereit.

     Diese Eingabe wird in einen String eingelesen und kann
     danach in die entsprechenden Variablentypen konvertiert
     werden.

     Fuer Kommazahlen wird eine Konvertierung in die eigene
     printf-Konvention mit Ersetzungszeichen %k oder in
     ein float bereitgestellt.

     Diese Software funktioniert aufgrund des 32-Bit Integer-
     wertes nur auf 32-Bit Mikrocontrollern

     MCU :

     08.04.2025  R. Seelig

     Hinweis:
       uart_readint muss in uart.h abgeschaltet sein
       (wird hier ersetzt)
   -------------------------------------------------------- */

#ifndef _in_uart_read
  #define _in_uart_read

  #include <stdio.h>
  #include <string.h>
  #include <stdlib.h>

  #include "uart.h"


  // Auswahl der Funktionen, die in den Code eingebunden werden sollen
  #define enable_int_convert     1
  #define enable_hex_convert     1
  #define enable_komma_convert   1
  #define enable_float_convert   1

  enum { stringinput, intinput, kommainput, hexinput };


  // Prototypen

  int my_atoi(char *str);
  int my_aftok(char *str, char *kommapos);
  float my_aftof(char *str);
  int my_ahtoi(char *str);
  char *uart_lineinput(char *string, uint8_t mode, char anz);

  // Funktionsmakros

  #define uart_readint(s, anz)               my_atoi(uart_lineinput(s, intinput, anz))
  #define uart_readhex(s, anz)               my_ahtoi(uart_lineinput(s, hexinput, anz))
  #define uart_readkomma(s, anz, kommapos)   my_aftok(uart_lineinput(s, kommainput, anz), kommapos)
  #define uart_readfloat(s, anz)             my_aftof(uart_lineinput(s, kommainput, anz))

#endif
